<?php namespace EwTools\Admin\ProjectPluginPage;

use EwTools\Storage\ObjectDB;
use EwTools\Support\Collection;
use EwTools\Support\Arr;
use EwTools\Support\Str;

/**
 * ComponentsPage
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin\ProjectPluginPage
 * @since   EwTools\Admin\ProjectPluginPage\ComponentsPage 0.1
 */
abstract class ComponentsPage {
    
    /**
     * Contains the all active plugins stored in WP
     *
     * @var
     */
    protected static $wp_active_plugins;
    
    /**
     * @var string
     */
    protected $text_domain = 'ewtools_textdomain';
    
    /**
     * Contains the widgets available for management
     *
     * @var array
     */
    protected $available_components;
    
    /**
     * Active plugins options
     *
     * @var string
     */
    protected static $option_name;
    
    /**
     * @var string
     */
    protected static $slug;
    
    /**
     * @var string
     */
    protected static $title;
    
    /**
     * Contains the path to the folder where the specified components are stored
     *
     * @var string
     */
    protected $components_path;
    
    /**
     * ComponentsPage constructor.
     *
     * @since 0.1
     *
     * @param $components_path
     */
    public function __construct( $components_path ) {
        
        $this->components_path = $components_path;
        
        /**
         * Hiding our own plugins for WordPress so they don't deactivate them.
         */
        //        add_filter( 'option_active_plugins', [ $this, 'hideComponents' ], 10, 1 );
        ObjectDB::onGet( 'active_plugins', [ $this, 'hideComponents' ] );
        
        /**
         * We are hooking in and adding our own plugins before saving the original plugins
         * A must do to first hide our plugins and the hook this on otherwise
         * WordPress will say our plugins is invalid and deactivate them.
         */
        //        add_filter( 'pre_update_option_active_plugins', [ $this, 'forceSaveComponents' ], 10, 1 );
        ObjectDB::beforeUpdate( 'active_plugins', [ $this, 'forceSaveComponents' ] );
    }
    // Filters/actions:
    // ======================================================
    
    /**
     * Hide plugins if we are on plugins.php so WordPress
     * don't deactivate them.
     *
     * @param array $plugins
     *
     * @return array
     */
    public function hideComponents( array $plugins = [] ) {
        static::$wp_active_plugins = $plugins;
        
        return $this->onWpPlugins() ? $this->getFilteredPlugins( $plugins ) : $plugins;
    }
    
    /**
     * To prepend our plugins to WordPress original update plugins
     *
     * @param array $plugins
     *
     * @return array
     */
    public function forceSaveComponents( array $plugins = [] ) {
        static::$wp_active_plugins = $this->getCombinedPlugins( $plugins );
        
        return static::$wp_active_plugins;
    }
    
    // Public functions:
    // ======================================================
    
    /**
     * Retrieve the slug to be use to identify the page
     *
     * @since 0.1
     * @return string
     */
    public function getSlug() {
        return Str::slug( static::$slug );
    }
    
    /**
     * Retrieve the title of the page to be used in the menu
     *
     * @since 0.1
     * @return string
     */
    public function getTitle() {
        return __( Str::title( static::$title ), $this->text_domain );
    }
    
    /**
     * Retrieve stored components from the options table and try to activate them.
     * Restores list after activation
     *
     * @since 0.1
     * @return bool
     */
    public function activateComponents() {
        return $this->storeComponents( array_filter( $this->activeComponents(), [ $this, 'activateComponent' ] ) );
    }
    
    /**
     * Activate component as a Wordpress plugin
     *
     * @param $component
     *
     * @return bool
     */
    public function activateComponent( $component ) {
        $file = WP_PLUGIN_DIR . '/' . $component;
        
        // Check so the plugin is available
        if( is_readable( $file ) ) {
            
            // Include file so activate can be triggered.
            include_once $file;
            
            do_action( 'activate_plugin', $component );
            do_action( "activate_{$component}" );
            do_action( 'activated_plugin', $component );
            
            return true;
        }
        
        return false;
    }
    
    /**
     * Use {add_submenu_page} to add the page to the plugin menu.
     *
     * @param string $parent_slug
     *
     * @since 0.1
     * @return string the menu hook_suffix
     */
    public function addToMenu( $parent_slug ) {
        $title = $this->getTitle();
        // Widgets
        $hook = add_submenu_page( $parent_slug, $title, $title, 'manage_options', Str::slug( "{$parent_slug}-{$title}" ), [
            $this,
            'renderPage'
        ] );
        
        return add_action( "load-{$hook}", [ $this, 'onPageLoad' ] );
    }
    
    /**
     * Retrieve a list of components from the specified folder
     * Used for fetching "Widgets" and "Plugins"
     *
     * @since 0.1
     * @return array
     */
    protected function availableComponents() {
        
        if( $this->available_components !== null ) {
            return $this->available_components;
        }
        $available_components = (array)get_plugins( "/{$this->components_path }" );
        
        foreach ( $available_components as $file => $component ) {
            $available_components[ $file ] = $this->createComponentObject( $component, "{$this->components_path}/{$file}" );
        }
        $this->available_components = $available_components;
        
        return $this->available_components;
    }
    
    /**
     * Create an object that can bu used in the admin list
     *
     * @param $component
     * @param $path
     *
     * @since 0.1
     * @return array
     */
    protected function createComponentObject( $component, $path ) {
        return array_replace( $component, [
            'Active' => in_array( $path, static::allPlugins(), true ),
            'Path'   => $path
        ] );
    }
    
    /**
     * Retrieve activated components from its options table
     *
     * @since 0.1
     * @return array
     */
    public function activeComponents() {
        $active_components = ObjectDB::get( static::$option_name, [] );
        
        return Arr::hasStringKeys( $active_components ) ? array_keys( $active_components ) : $active_components;
    }
    
    /**
     * Deactivate all active components
     *
     * @since 0.1
     * @return void
     */
    public function deactivateComponents() {
        Collection::make( $this->activeComponents() )->each( [ $this, 'deactivateComponent' ] );
    }
    
    /**
     * Deactivate component from wordpress plugins
     *
     * @param $component
     *
     * @since 0.1
     * @return void
     */
    public function deactivateComponent( $component ) {
        do_action( 'deactivate_plugin', $component );
        do_action( "deactivate_{$component}" );
        do_action( 'deactivated_plugin', $component );
    }
    
    /**
     * Return the filename from an Everyware component
     *
     * @param $component
     *
     * @since 0.1
     * @return string
     */
    public function getFileName( $component ) {
        if( Str::startsWith( $component, $this->components_path ) ) {
            return Str::substr( $component, Str::length( Str::finish( $this->components_path, '/' ) ) );
        }
        
        return $component;
    }
    
    /**
     * Retrieve a merged list of active plugins and Everyware components
     *
     * @param array $plugins
     *
     * @since 0.1
     * @return array
     */
    public function getCombinedPlugins( $plugins = null ) {
        return array_unique( array_merge( $this->getFilteredPlugins( $plugins ), $this->activeComponents() ) );
    }
    
    /**
     * Retrieve a list of plugins from Wordpress options table and filter out the components
     *
     * @since 0.1
     *
     * @param array $plugins
     *
     * @return array
     */
    public function getFilteredPlugins( $plugins = null ) {
        return array_filter( $plugins ?: static::allPlugins(), [ $this, 'notComponent' ] );
    }
    
    /**
     * Check if a value is a Component added by Everyware
     *
     * @param string $component
     *
     * @since 0.1
     * @return bool
     */
    public function isComponent( $component ) {
        return Str::startsWith( $component, $this->components_path );
    }
    
    /**
     * Check if a value is a Component added by Everyware
     *
     * @param string $component
     *
     * @since 0.1
     * @return bool
     */
    public function notComponent( $component ) {
        return ! $this->isComponent( $component );
    }
    
    /**
     * Function that fires on page load
     *
     * @since 0.1
     * @return void
     */
    public function onPageLoad() {
        
        // Update active plugins with the others
        if( $this->onComponentsUpdate() && $this->updateActiveState() ) {
            ObjectDB::update( 'active_plugins', static::allPlugins() );
        }
    }
    
    /**
     * Activate/Deactivate components according to the new list
     *
     * @since 0.1
     * @return bool
     */
    public function updateActiveState() {
        $active_components = $this->activeComponents();
        $new_components    = $this->getSubmittedComponents();
        
        // Deactivate components that isn't in the new active list
        foreach ( array_keys( $active_components ) as $component ) {
            if( ! array_key_exists( $component, $new_components ) ) {
                $this->deactivateComponent( $component );
            }
        }
        
        // Activate components that isn't in the old active list
        foreach ( array_keys( $new_components ) as $component ) {
            // Check if widgets exist if so activate it
            if( ! array_key_exists( $component, $active_components ) && ! $this->activateComponent( $component ) ) {
                unset( $new_components[ $component ] );
            }
        }
        
        return $this->storeComponents( $new_components );
    }
    
    // Protected Functions:
    // ======================================================
    
    /**
     * Retrieve component page specific data with added data to handle management of component-list
     *
     * @param array $custom_data
     *
     * @since 0.1
     * @return array
     */
    protected function getComponentsPageData( array $custom_data = [] ) {
        $current_list_status_filter = $this->getCurrentStatusFilter();
        $view_data                  = array_replace_recursive( [
            'component_type'    => $this->getTitle(),
            'description'       => __( 'From here you can manage your Everyware components for the site.', $this->text_domain ),
            'updated'           => $this->onComponentsUpdate(),
            'plugin_status'     => $current_list_status_filter,
            'plugin_status_url' => [
                'all'      => add_query_arg( [ 'plugin_status' => 'all' ] ),
                'active'   => add_query_arg( [ 'plugin_status' => 'active' ] ),
                'inactive' => add_query_arg( [ 'plugin_status' => 'inactive' ] )
            ],
            'form_action'       => add_query_arg( [ 'settings-updated' => 'true' ] ),
            'text_domain'       => $this->text_domain,
            'components'        => $this->availableComponents(),
            'active_components' => []
        ], $custom_data );
        
        $components = new Collection( $view_data[ 'components' ] );
        
        $view_data[ 'total_count' ]    = $components->count();
        $view_data[ 'active_count' ]   = $components->where( 'Active', true )->count();
        $view_data[ 'inactive_count' ] = $components->where( 'Active', false )->count();
        
        return $view_data;
    }
    
    /**
     * Retrieve the current status-filter of the components-list
     *
     * @since 0.1
     * @return string
     */
    protected function getCurrentStatusFilter() {
        return isset( $_GET[ 'plugin_status' ] ) ? $_GET[ 'plugin_status' ] : 'all';
    }
    
    /**
     * Check if components management page have posted changes
     *
     * @since 0.1
     * @return bool
     */
    protected function onComponentsUpdate() {
        return isset( $_POST[ 'everyware-component-submit' ] );
    }
    
    /**
     * Check if on plugins.php aka Wordpress plugins management
     *
     * @since 0.1
     * @return bool
     */
    protected function onWpPlugins() {
        global $pagenow;
        
        return $pagenow === 'plugins.php';
    }
    
    /**
     * Update specific Components in options table
     *
     * @param array $components
     *
     * @since 0.1
     * @return bool
     */
    protected function storeComponents( array $components = [] ) {
        return ObjectDB::update( static::$option_name, $components );
    }
    
    /**
     * Retrieve new list of components posted by the admin page
     *
     * @since 0.1
     * @return array
     */
    protected function getSubmittedComponents() {
        return array_filter( (array)array_get( $_POST, 'active_components', [] ) );
    }
    
    // Abstract Functions: Functions that must be implemented by child-classes
    // ======================================================
    
    /**
     * Function for outputting the admin page of the components from the menu
     *
     * @since 0.1
     * @return mixed
     */
    abstract public function renderPage();
    
    // Static Functions:
    // ======================================================
    /**
     * Retrieve all "Plugins" activated in Wordpress from the options table
     *
     * @since 0.1
     * @return mixed
     */
    protected static function allPlugins() {
        if( static::$wp_active_plugins === null ) {
            static::$wp_active_plugins = ObjectDB::get( 'active_plugins', [] );
        }
        
        return static::$wp_active_plugins;
    }
}