<?php
/**
 * Handler: for managing the body of an article
 */

namespace EwTools\Handler;

use EwTools\Support\Str;
use SimpleXMLIterator;

/**
 * ArticleBody
 *
 * @link    http://infomaker.se
 * @package Everyware\Utilities
 * @since   Everyware\Utilities\ArticleBody 0.1
 */
class ArticleBody {
    
    /**
     * Contains the body in the form of a list blocks
     *
     * @var array
     */
    protected $body = [];
    
    /**
     * Contains a list of block elements allowed to pass through the filter
     *
     * @var array
     */
    protected static $allowed_blocks = [
        'h1',
        'h2',
        'h3',
        'h4',
        'h5',
        'h6',
        'p',
        'div',
        'blockquote',
        'subheadline1',
        'subheadline2',
        'subheadline3',
        'subheadline4',
        'subheadline5',
        'subheadline6'
    ];
    
    /**
     * Contains a list of inline elements allowed to pass through the filter
     *
     * @var array
     */
    protected static $allowed_inlines = [
        'span',
        'i',
        'b',
        'em',
        'strong',
        'sup',
        'sub',
        'br',
        'hr',
        'a',
        'script',
        'iframe'
    ];
    
    public function __construct( $body = '' ) {
        $this->setBody( $body );
    }
    
    /**
     * Retrieve the number of blocks in the body
     *
     * @since 0.1
     * @return int
     */
    public function count() {
        return count( $this->body );
    }
    
    /**
     * Retrieve the body text in the form of an array
     *
     * @since 0.1
     * @return string
     */
    public function toString() {
        return implode( '', $this->body );
    }
    
    /**
     * Retrieve the body text in the form of an string
     *
     * @since 0.1
     * @return array
     */
    public function toArray() {
        return $this->body;
    }
    
    /**
     * Retrieve a merged list of all allowed tags in the text
     *
     * @since 0.1
     * @return array
     */
    public function getAllowedTags() {
        return array_merge( static::$allowed_blocks, static::$allowed_inlines );
    }
    
    /**
     * Add a block to the body text
     *
     * @param string $block
     * @param string $block_tag
     *
     * @since 0.1
     * @return void
     */
    public function addBlock( $block, $block_tag = '' ) {
        if( ! Str::isEmpty( $block_tag ) ) {
            $block = "<{$block_tag}>{$block}</$block_tag>";
        }
        
        if( ( $block = $this->onBlockAdd( $block, $block_tag ) ) !== false ) {
            $this->body[] = $block;
        }
    }
    
    /**
     * Converts one specified tag into another with the option to add attributes to the new tag
     *
     * @param string $tag
     * @param string $replacement
     * @param string $subject
     * @param array  $attributes
     *
     * @since 0.1
     * @return string|null
     */
    protected function convertTag( $tag, $replacement, $subject, $attributes = [] ) {
        $tag_attr = '';
        
        foreach ( $attributes as $name => $value ) {
            $tag_attr .= " {$name}=" . '"' . $value . '"';
        }
        
        return str_replace( [ "<{$tag}>", "</{$tag}>" ], [
            "<{$replacement}{$tag_attr}>",
            "</{$replacement}>"
        ], $subject );
    }
    
    /**
     * Filter the content of the blocks from unwanted elements.
     * This function will be applied to every block
     *
     * @param string $text
     *
     * @since 0.1
     * @return string
     */
    protected function filterInlineContent( $text ) {
        return trim( strip_tags( $text, '<' . implode( '><', static::$allowed_inlines ) . '>' ) );
    }
    
    /**
     * Filter that fires before the block is added needs to return the block that will be added
     * If false is returned the block will not be added
     *
     * @param string $content
     * @param string $current_tag
     *
     * @since 0.1
     * @return string
     */
    protected function onBlockAdd( $content, $current_tag = '' ) {
        $content = preg_replace( '/<\/subheadline(\d{1})>/', '</h3>', $content );
        $content = preg_replace( '/<subheadline(\d{1})>/', '<h3 class="subheadline$1">', $content );
        
        return $content;
    }
    
    /**
     * @param $string
     *
     * @since 0.1
     * @return string
     */
    protected function removeLinebreak( $string ) {
        return str_replace( [ "\r", "\n" ], '', $string );
    }
    
    /**
     * Filter and remove unwanted elements and adds valid blocks
     *
     * @param string $body
     *
     * @since 0.1
     * @return void
     */
    protected function setBody( $body = '' ) {
        if( Str::startsWith( $body, '<body xmlns="http://www.infomaker.se/npdoc/' ) ) {
            $this->setFromXML( $body );
        } else {
            $this->setFromString( $body );
        }
    }
    
    /**
     * Filter and remove unwanted elements and adds valid string of blocks
     *
     * @param string $str
     *
     * @since 0.1
     * @return void
     */
    public function setFromString( $str ) {
        $str = strip_tags( $str, '<' . implode( '><', $this->getAllowedTags() ) . '>' );
        
        preg_match_all( '/<([a-z][a-z0-9]*)\b[^>]*>(.*?)<\/\1>/sim', $str, $html_tags );
        list( $tags, $tags_types, $tag_content ) = $html_tags;
        
        foreach ( (array)$tag_content as $type => $content ) {
            $tag = $tags_types[ $type ];
            $content = $this->filterInlineContent( $content );
            $content = $this->removeLinebreak( $content );
            if( ! Str::isEmpty( $content ) && in_array( $tag, static::$allowed_blocks, true ) ) {
                $this->addBlock( $content, $tag );
            }
        }
    }
    
    /**
     * Filter and remove unwanted elements and adds valid blocks
     *
     * @param string $xml
     *
     * @since 0.1
     * @return void
     */
    public function setFromXML( $xml ) {
        $text = new SimpleXMLIterator( $xml );
        
        foreach ( $text as $tag => $content ) {
            $content = $this->filterInlineContent( $content->asXML() );
            $content = $this->removeLinebreak( $content );
            if( ! Str::isEmpty( $content ) && in_array( $tag, static::$allowed_blocks, true ) ) {
                $this->addBlock( $content, $tag );
            }
        }
    }
    
    /**
     * Quick way to retrieve the article body in the form of an array of blocks
     *
     * @param string $body
     *
     * @since 0.1
     * @return array
     */
    public static function getBlockList( $body ) {
        return ( new static( $body ) )->toArray();
    }
    
    /**
     * Quick way to retrieve the article body in the form of a string
     *
     * @param string $body
     *
     * @since 0.1
     * @return string
     */
    public static function getBlockString( $body ) {
        return ( new static( $body ) )->toString();
    }
}