<?php namespace EwTools\Handler;

use EwTools\Support\Str;

/**
 * FeedRouter
 *
 * @link    http://infomaker.se
 * @package EwTools
 * @since   EwTools\FeedRouter 0.1
 */
class FeedRouter {
    
    /**
     * The file where the feed-routes are registered and handled
     *
     * @var string
     */
    protected static $feed_file = 'feed.php';
    
    /**
     * Contains the registered types of feed templates
     *
     * @var array
     */
    protected static $registered_routes = [];
    
    /**
     * Contains the name of the GET-parameter where the route is stored
     *
     * @var string
     */
    protected static $route_parameter = 'route';
    
    /**
     * Contains all supported types of feed
     *
     * @var array
     */
    protected static $valid_feeds = [ 'rss', 'rss2', 'atom', 'rdf' ];
    
    /**
     * Singleton container
     *
     * @var FeedRouter
     */
    private static $instance;
    
    /**
     * Contains the type of the requested feed
     *
     * @var string Type of feed.
     */
    protected $feed_type;
    
    /**
     * Contains the requested route
     *
     * @var string
     */
    protected $route;
    
    /**
     * Constructor
     */
    private function __construct() {
    }
    
    /**
     * Load the feed-template file and route to a registered route
     *
     * @since 0.1
     * @return void
     */
    public function loadFeedFile() {
        locate_template( static::$feed_file, true );
        $route     = '/' . trim( array_get( $_GET, static::$route_parameter, '' ), '/' );
        $feed_type = array_get( $_GET, 'feed', 'rss' );
        $this->routeRequest( $route, $feed_type );
    }
    
    /**
     * Add a route to match with
     *
     * @param $route
     * @param $callback
     *
     * @since 0.1
     * @return void
     */
    public function addRoute( $route, $callback ) {
        static::$registered_routes[ '/' . trim( $route, '/' ) ] = $callback;
    }
    
    /**
     * Match route with registered patterns
     *
     * @param string $route
     *
     * @since 0.1
     * @return mixed
     */
    public function getMatchedPattern( $route ) {
        
        // Check if no exact match exist
        if( ! array_key_exists( $route, static::$registered_routes ) ) {
            
            foreach ( array_keys( static::$registered_routes ) as $registered_route ) {
                
                if( $this->matchesRoutePattern( $route, $registered_route ) ) {
                    return $registered_route;
                }
                
                if( Str::is( $registered_route, $route ) ) {
                    return $registered_route;
                }
            }
        }
        
        return $route;
    }
    
    /**
     * Check for match among routs with optional parameters or
     * check for match with wildcard route
     *
     * @param string $pattern
     * @param string $route
     *
     * @since 0.1
     * @return bool
     */
    protected function matchesRoutePattern( $route, $pattern ) {
        return (bool)preg_match( $this->createRegExpPattern( $pattern ), $route ) || Str::is( $pattern, $route );
    }
    
    /**
     * Create a regular expression to match routes with optional parameters
     *
     * @param string $route_pattern
     *
     * @since 0.1
     * @return string
     */
    protected function createRegExpPattern( $route_pattern ) {
        return '/^' . str_replace( '/', '\/', preg_replace( '/\{([\w-]+?)\}/', '([\w-]+?)', $route_pattern ) ) . '$/';
    }
    
    /**
     * Get the values of the optional parameters from the route.
     *
     * @param $route
     * @param $pattern
     *
     * @since 0.1
     * @return array
     */
    protected function extractOptionalParameters( $route, $pattern ) {
        if( $pattern === '/*' ) {
            return explode( '/', trim( $route, '/' ) );
        }
    
        preg_match_all( $this->createRegExpPattern( $pattern ), $route, $matches );
    
        return isset( $matches[ 1 ] ) ? array_flatten( array_slice( $matches, 1 ) ) : [];
    }
    
    /**
     * Route the requested feed by finding a match from the list of registered route-patterns
     * and sending the optional parameters
     *
     * @param string $route
     * @param string $feed_type
     *
     * @since 0.1
     * @return void
     */
    protected function routeRequest( $route, $feed_type = 'rss' ) {
        $route_parts   = explode( '/', trim( $route, '/' ) );
    
        // Decode and slugify every parameter before proceeding
        $decoded_route = '/' . implode( '/', array_map( function ( $part ) {
                return Str::slug( urldecode( $part ) );
            }, $route_parts ) );
    
        $matched_pattern = $this->getMatchedPattern( $decoded_route );
        $params          = $this->extractOptionalParameters( $decoded_route, $matched_pattern );
    
        // Filter original route_parts to only keep the requested params
        $filtered_params = array_filter($route_parts, function ( $part ) use ($params) {
            return in_array( Str::slug( urldecode( $part )), $params, true );
        });
    
        $this->callRoute( $matched_pattern, $feed_type, $filtered_params );
    }
    
    /**
     * Call registered function with matching parameters
     *
     * @param string $route
     * @param string $type
     * @param array $params
     *
     * @since 0.1
     * @return void
     */
    protected function callRoute( $route, $type, array $params = [] ) {
        if( array_key_exists( $route, static::$registered_routes ) ) {
            header( 'Content-Type: ' . feed_content_type( $type ) . '; charset=' . get_option( 'blog_charset' ), true, 200 );
            call_user_func_array( static::$registered_routes[ $route ], $params );
        }
    }
    
    /**
     * Retrieve instance of Singleton
     *
     * @since 0.1
     * @return FeedRouter
     */
    public static function getInstance() {
        if( static::$instance === null ) {
            static::$instance = new static;
        }
        
        return static::$instance;
    }
    
    /**
     * Initiate FeedRouter
     *
     * @since 0.1
     * @return FeedRouter
     */
    public static function init() {
        
        $router = static::getInstance();
        foreach ( static::$valid_feeds as $feed ) {
            // Remove WP-action
            remove_all_actions( "do_feed_{$feed}" );
            // Route to feed-file
            add_action( "do_feed_{$feed}", [ $router, 'loadFeedFile' ], 10, 1 );
        }
        
        /** Change Rss url */
        remove_action( 'wp_head', 'feed_links_extra', 3 ); // Displays the links to the extra feeds such as category feeds
        remove_action( 'wp_head', 'feed_links', 2 ); // Displays the links to the general feeds: Post and Comment Feed
        
        return $router;
    }
    
    /**
     * Helper-function to add a route to match for
     *
     * @param string   $route
     * @param callable $callback
     *
     * @since 0.1
     * @return void
     */
    public static function register( $route, callable $callback ) {
        static::getInstance()->addRoute( $route, $callback );
    }
}