<?php namespace EwTools\Handler;

use EwTools\Support\Str;

/**
 * Forms
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\Forms 0.1
 */
class Forms {
    
    /**
     * Contains the prefix to be added to every form item
     *
     * @var string
     */
    private $field_prefix;
    
    /**
     * Contains a list of fields with with default values
     *
     * @var array
     */
    private $fields;
    
    /**
     * Forms constructor.
     *
     * @since 0.1
     *
     * @param string $field_prefix
     * @param array  $fields
     */
    public function __construct( $field_prefix, array $fields = [] ) {
        
        $this->field_prefix = Str::slug( $field_prefix );
        $this->fields       = $fields;
    }
    
    /**
     * @param array $data
     *
     * @since 1.0.0
     * @return array
     */
    public function getFormData( array $data = [] ) {
        return $this->setupFields( array_replace_recursive( $this->fields, $data ) );
    }
    
    /**
     * Function to setup field-values for the view
     *
     * @param array $instance
     *
     * @since 0.1
     * @return array
     */
    protected function setupFields( array $instance = [] ) {
        $fields_values = [];
        
        foreach ( $this->fields as $field => $value ) {
            $fields_values[ $field ] = is_array( $value ) ? $this->getFieldMultiValues( $field, $instance[ $field ] ) : $this->getFieldValues( $field, $instance[ $field ] );
        }
        
        return $fields_values;
    }
    
    /**
     * Return an array with the id and name from a multi valued field for the form input
     *
     * @param string $field
     * @param array $instance
     *
     * @since 0.1
     * @return array
     */
    protected function getFieldMultiValues( $field, array $instance = []) {
        $return_val = [];
        foreach ( $instance as $i => $value ) {
            
            if( ! is_array( $value ) ) {
                return $this->getFieldValues( "{$field}[]", $instance );
            }
            
            foreach ( (array)$value as $settings_name => $settings_value ) {
                $return_val[ $i ][ $settings_name ] = $this->getFieldValues( "{$field}[{$i}][{$settings_name}]", $instance[ $i ][ $settings_name ] );
            }
        }
        
        return $return_val;
    }
    
    /**
     * Return an array with the id and name for the form input
     *
     * @param string $field
     * @param mixed $value
     *
     * @since 0.1
     * @return array
     */
    protected function getFieldValues( $field, $value ) {
        
        return [
            'id'    => $this->getFieldId( $field ),
            'name'  => $this->getFieldName( $field ),
            'value' => $value
        ];
    }
    
    /**
     * Create a field-id used on form-input
     *
     * @param string $field_name
     *
     * @since 0.1
     * @return string
     */
    protected function getFieldId( $field_name ) {
        return $this->field_prefix . '-' . trim( str_replace( [ '[]', '[', ']' ], [
                '',
                '-',
                ''
            ], $field_name ), '-' );
    }
    
    /**
     * Create a field-name used on form-input
     *
     * @param string $field_name
     *
     * @since 0.1
     * @return string
     */
    protected function getFieldName( $field_name ) {
        if( false === $pos = strpos( $field_name, '[' ) ) {
            return $this->field_prefix . "[{$field_name}]";
        }
        
        return $this->field_prefix . '[' . substr_replace( $field_name, '][', $pos, strlen( '[' ) );
    }
}