<?php
/**
 * Model: Represents a menu
 */

namespace EwTools\Models;

use EwTools\Support\Collection;
use EwTools\Support\Fluent;
use WP_Term;

/**
 * Menu
 *
 * @link    http://infomaker.se
 * @package Everyware\Models
 * @since   Everyware\Models\Menu 0.1
 */
class Menu extends Fluent {
    
    /**
     * Contains the defaults for fetching menus via WP
     *
     * @var array
     */
    protected static $defaults = [
        'taxonomy'               => 'nav_menu',
        'orderby'                => 'name',
        'order'                  => 'ASC',
        'hide_empty'             => false,
        'include'                => [],
        'exclude'                => [],
        'exclude_tree'           => [],
        'number'                 => '',
        'offset'                 => '',
        'fields'                 => 'all',
        'name'                   => '',
        'slug'                   => '',
        'hierarchical'           => true,
        'search'                 => '',
        'name__like'             => '',
        'description__like'      => '',
        'pad_counts'             => false,
        'get'                    => '',
        'child_of'               => 0,
        'parent'                 => '',
        'childless'              => false,
        'cache_domain'           => 'core',
        'update_term_meta_cache' => true,
        'meta_query'             => ''
    ];
    
    /**
     * Contains a list of items in the menu
     *
     * @var array
     */
    protected $items = [];
    
    /**
     * Menu constructor.
     *
     * @since 0.1
     *
     * @param array $menu
     */
    public function __construct( $menu = [] ) {
        if( $menu instanceof WP_Term ) {
            $menu = $menu->to_array();
        }
        parent::__construct( $menu );
    }
    
    /**
     * Retrieve the menu generated into a html string
     *
     * @param  $options
     *
     * @since 0.1
     * @return false|object|string
     */
    public function generate( $options ) {
        return wp_nav_menu( array_replace_recursive( $options, [
            'menu' => $this->term_id,
            'echo' => false
        ] ) );
    }
    
    /**
     * Retrieve menu items with option to build an hierarchical tree-structure
     *
     * @param bool $build_tree
     *
     * @since 0.1
     * @return Collection
     */
    public function items( $build_tree = false ) {
        $items = [];
        if( false !== ( $menu_items = wp_get_nav_menu_items( $this->term_id ) ) ) {
            
            $items = array_map( function ( $menu_item ) {
                return (array)$menu_item;
            }, $menu_items );
            
            $items = $build_tree ? $this->buildTree( $items ) : $items;
        }
        
        return new Collection( $items );
    }
    
    /**
     * @param string $mark
     * @param bool   $build_tree
     *
     * @since 0.1
     * @return Collection
     */
    public function markedItems( $mark, $build_tree = false ) {
        return array_map( function ( $item ) use ( $mark ) {
            $classes           = array_filter( $item[ 'classes' ] );
            $classes[]         = $mark;
            $item[ 'classes' ] = $classes;
            
            return $item;
        }, $this->items( $build_tree )->toArray() );
    }
    
    /**
     * Render the menu
     *
     * @param $options
     *
     * @since 0.1
     * @return void
     */
    public function render( $options ) {
        wp_nav_menu( array_replace_recursive( $options, [
            'menu' => $this->term_id,
            'echo' => true
        ] ) );
    }
    
    /**
     * Build an hierarchical tree out of an array of menu-items
     *
     * @param array $elements
     * @param int   $parentId
     *
     * @since 0.1
     * @return Collection
     */
    protected function buildTree( $elements = [], $parentId = 0 ) {
        $branch = [];
        
        foreach ( $elements as $element ) {
            if( (int)$element[ 'menu_item_parent' ] === $parentId ) {
                $element[ 'children' ] = $this->buildTree( $elements, $element[ 'ID' ] );
                $branch[]              = $element;
            }
        }
        
        return new Collection( $branch );
    }
    
    /**
     * Retrieves all menus
     *
     * @param array $args Optional. Array of arguments passed on to {@see get_terms()}.
     *                    Default empty array.
     *
     * @since 0.1
     * @return Collection
     */
    public static function all( $args = [] ) {
        $menus = wp_get_nav_menus( array_replace_recursive( static::$defaults, $args ) );
        
        $menu_collection = [];
        if( is_array( $menus ) ) {
            $menu_collection = array_map( function ( WP_Term $menu ) {
                return new static( $menu );
            }, $menus );
        }
        
        return new Collection( $menu_collection );
    }
}