<?php namespace EwTools\Storage;

use EwTools\Interfaces\Support\Arrayable;
use EwTools\Interfaces\Support\Jsonable;
use EwTools\Support\Collection;
use EwTools\Support\Fluent;
use EwTools\Support\Str;

/**
 * CollectionModel
 *
 * @link    http://infomaker.se
 * @package EwTools\Models
 * @since   EwTools\Models\CollectionModel 0.1
 */
class CollectionModel extends Fluent {
    
    /**
     * Contains the name of the options table associated with the model
     *
     * @var string
     */
    protected $option_name;
    
    /**
     * Describes whether the collection can be reached across the network of sites
     *
     * @var bool
     */
    protected $global = false;
    
    /**
     * Fields that will be set and stored
     *
     * @var array
     */
    protected $fields = [];
    
    /**
     * CollectionModel constructor.
     *
     * @since 0.1
     *
     * @param array $attributes
     */
    public function __construct( $attributes = [] ) {
        $valid_fields = $this->getFields( $attributes );
        parent::__construct( $this->onSet( $valid_fields ) );
    }
    
    /**
     * Results array of items from CollectionModel or Arrayable.
     *
     * @param mixed $items
     *
     * @since 0.1
     * @return array
     */
    protected function getArrayableItems( $items ) {
        if ( is_array( $items ) ) {
            return $items;
        } elseif ( $items instanceof self ) {
            return $items->getAttributes();
        } elseif ( $items instanceof Arrayable ) {
            return $items->toArray();
        } elseif ( $items instanceof Jsonable ) {
            return json_decode( $items->toJson(), true );
        }
        
        return (array) $items;
    }
    
    /**
     * Filter attributes to only use the filed specified in the "fields property"
     *
     * @param array $attributes
     *
     * @since 0.1
     * @return array
     */
    protected function getFields( $attributes = [] ) {
        $attributes = $this->getArrayableItems( $attributes );
        
        if( empty( $this->fields ) ) {
            return $attributes;
        }
        
        // Add defaults to make sure all fields are set
        $attributes = array_replace_recursive( $this->fields, $attributes );
        
        $valid_fields = array_keys( $this->fields );
        
        // Filter out field that are not specified in the "fields property"
        return array_filter( $attributes, function ( $key ) use ( $valid_fields ) {
            return in_array( $key, $valid_fields, true );
        }, ARRAY_FILTER_USE_KEY );
    }
    
    /**
     * Connect to the CollectionDB table
     *
     * @since 0.1
     * @return CollectionDB
     */
    public function newCollection() {
        return new CollectionDB( $this->option_name, $this->global );
    }
    
    /**
     * Function that's called before fields are set as properties
     *
     * @param array $fields
     *
     * @since 0.1
     * @return array
     */
    protected function onSet( $fields = [] ) {
        return $fields;
    }
    
    /**
     * Save current model in collection
     *
     * @since 0.1
     * @return bool
     */
    public function save() {
        return $this->newCollection()->add( $this )->save();
    }
    
    /**
     * Get all of the model from the database.
     *
     * @since 0.1
     * @return Collection
     */
    public static function all() {
        $instance = new static();
        
        return $instance->newCollection()->all()->map( function ( $item ) {
            return new static( $item );
        } );
    }
    
    /**
     * Handle Collection table
     *
     * @since 0.1
     * @return CollectionDB
     */
    public static function collection() {
        return ( new static )->newCollection();
    }
    
    /**
     * Another way of creating and saving an collection item
     *
     * @param array $attributes
     *
     * @since 0.1
     * @return static
     */
    public static function create( array $attributes = [] ) {
        $model = new static( $attributes );
        $model->save();
        
        return $model;
    }
    
    /**
     * Find a specific value from the collection
     *
     * @param $key
     *
     * @since 0.1
     * @return mixed
     */
    public static function find( $key ) {
        return static::all()->pull( $key );
    }
    
    /**
     * Update
     *
     * @param string $key
     * @param array  $item
     *
     * @since 0.1
     * @return void
     */
    public static function update( $key, $item = [] ) {
        $instance = $item instanceof self ? $item : new static( $item );
        $instance->newCollection()->update( $key, $instance );
    }
    
    /**
     * Filter items by the given key value pair.
     *
     * @param string $key
     * @param string $value
     * @param bool   $strict
     *
     * @since 0.1
     * @return Collection
     */
    public static function where( $key, $value, $strict = true ) {
        return static::makeCollection( static::collection()->all()->where( $key, $value, $strict ) );
    }

    /**
     * Filter items by the given key value pair in depth of Collection
     *
     * @param string $key
     * @param string $value
     * @param bool   $strict
     *
     * @since 0.1.10
     * @return Collection
     */
    public static function whereIn( $key, $value, $strict = true ) {
        return static::makeCollection( static::collection()->all()->whereIn( $key, $value, $strict ) );
    }
    
    /**
     * Create a valid collection
     * Has option to use an item-property as collection key
     *
     * @param array  $items
     * @param string $use_as_key
     *
     * @since 0.1
     * @return Collection
     */
    public static function makeCollection( $items = [], $use_as_key = null ) {
        
        if( ! $use_as_key ) {
            return Collection::make( $items )->map( function ( $item ) {
                return new static( $item );
            } );
        }
        
        $collection = [];
        foreach ( $items as $key => $item ) {
            if( isset( $item[ $use_as_key ] ) && Str::notEmpty( $item[ $use_as_key ] ) ) {
                $key = Str::slug($item[ $use_as_key ]);
            }
            $collection[ $key ] = new static( $item );
        }
        
        return new Collection( $collection );
    }
}