<?php
/**
 * Storage handler: for handling document-database functionality by using Wordpress options table
 */

namespace EwTools\Storage;

use EwTools\Interfaces\Storage\WpOptionsInterface;
use EwTools\Support\Str;

/**
 * ObjectDB
 *
 * @link    http://infomaker.se
 * @package EwTools\Storage
 * @since   EwTools\Storage\ObjectDB 0.1
 */
class ObjectDB implements WpOptionsInterface {
    
    /**
     * Add data to document-database
     *
     * @see   add_option()
     *
     * @param string $option
     * @param mixed  $value
     * @param bool   $autoload
     *
     * @since 0.1
     * @return bool
     */
    public static function add( $option, $value, $autoload = true ) {
        return add_option( $option, $value, '', $autoload ? 'yes' : 'no' );
    }
    
    /**
     * Add global data to document-database
     *
     * @see   add_network_option()
     *
     * @param int    $network_id
     * @param string $option
     * @param mixed  $value
     *
     * @since 0.1
     * @return bool
     */
    public static function addGlobal( $network_id, $option, $value ) {
        return add_network_option( $network_id, $option, $value );
    }
    
    /**
     * Retrieve data from document-database
     *
     * @see   get_option()
     *
     * @param string $option
     * @param mixed  $default
     *
     * @since 0.1
     * @return mixed
     */
    public static function get( $option, $default = false ) {
        return get_option( $option, $default );
    }
    
    /**
     * Retrieve global data from document-database
     *
     * @see   get_network_option()
     *
     * @param int    $network_id
     * @param string $option
     * @param bool   $default
     *
     * @since 0.1
     * @return mixed
     */
    public static function getGlobal( $network_id, $option, $default = false ) {
        return get_network_option( $network_id, $option, $default );
    }
    
    /**
     * Remove data from document-database
     *
     * @see   delete_option()
     *
     * @param string $option
     *
     * @since 0.1
     * @return bool
     */
    public static function remove( $option ) {
        return delete_option( $option );
    }
    
    /**
     * Remove global data from document-database
     *
     * @see   delete_network_option()
     *
     * @param int    $network_id
     * @param string $option
     *
     * @since 0.1
     * @return bool
     */
    public static function removeGlobal( $network_id, $option ) {
        return delete_network_option( $network_id, $option );
    }
    
    /**
     * Update data on document-database
     *
     * @see   update_option()
     *
     * @param string $option
     * @param mixed  $value
     * @param bool   $autoload
     *
     * @since 0.1
     * @return bool
     */
    public static function update( $option, $value, $autoload = true ) {
        return update_option( $option, $value, $autoload ? 'yes' : 'no' );
    }
    
    /**
     * Update global data on document-database
     *
     * @see   update_network_option()
     *
     * @param int    $network_id
     * @param string $option
     * @param mixed  $value
     *
     * @since 0.1
     * @return mixed
     */
    public static function updateGlobal( $network_id, $option, $value ) {
        return update_network_option( $network_id, $option, $value );
    }
    
    /**
     * Filters the value of an existing option before it is retrieved.
     * Passing a truthy value to the filter will short-circuit retrieving
     * the option value, returning the passed value instead.
     *
     * @see   get_option()
     * Args passed to callback : {
     * @var bool|mixed $pre_option Value to return instead of the option value.
     *                               Default false to skip it.
     * @var string     $option     Option name.
     * }
     *
     * @param string   $option
     * @param callable $callback
     * @param int      $priority
     *
     * @since 0.1
     * @return void
     */
    public function beforeGet( $option, callable $callback, $priority = 10 ) {
        add_filter( "pre_option_{$option}", $callback, $priority, 2 );
    }
    
    /**
     * Filters the value of an existing option.
     * Args passed to callback : {
     *
     * @var mixed      $value  Value of the option. If stored serialized, it will be
     *                       unserialized prior to being returned.
     * @var string     $option Option name.
     * }
     *
     * @param string   $option
     * @param callable $callback
     * @param int      $priority
     *
     * @return void
     */
    public static function onGet( $option, callable $callback, $priority = 10 ) {
        add_filter( "option_{$option}", $callback, $priority, 2 );
    }
    
    /**
     * Filters an option before its value is (maybe) serialized and updated.
     * Optional to choose specific option
     *
     * @see   update_option()
     * Args passed to callback : {
     * @var mixed      $value     The new, unserialized option value.
     * @var mixed      $old_value The old option value.
     * @var string     $option    Option name.
     * }
     *
     * @param string   $option
     * @param callable $callback
     * @param int      $priority
     *
     * @since 0.1
     * @return void
     */
    public static function beforeUpdate( $option = null, callable $callback, $priority = 10 ) {
        $filter_tag = $option === null ? 'pre_update_option' : "pre_update_option_{$option}";
        add_filter( $filter_tag, $callback, $priority, 3 );
    }
}
